@@ -5,7 +5,7 @@ from time import time,sleep
 from PyQt5.QtWidgets import QAction
 from PyQt5 import  QtGui
 import imageio
-from pyFBS.utility import *
+from .utility import *
 import keyboard as kb
 from scipy.spatial.transform import Rotation as R
 from pathlib import Path
@@ -211,12 +211,18 @@ class view3D():
             add_val = ann[:, :, i]
             add_val_secondary = ann_secondary[:, i]
 
-            self.plot.update_coordinates(self.modeshape_animation["or_pts"] + add_val, mesh=self.modeshape_animation["mesh"],render = False)
-            if self.modeshape_animation["scalars"]:
-                if self.modeshape_animation["animate_secondary_mode_shape"]==True:
-                    self.plot.update_scalars(add_val_secondary, mesh=self.modeshape_animation["mesh"] ,render = False) # for color changing
-                else: 
-                    self.plot.update_scalars(np.sqrt(np.mean(add_val ** 2, axis=1)).reshape(self.modeshape_animation["or_pts"].shape[0]), mesh=self.modeshape_animation["mesh"] ,render = False) # for color changing
+            if type(self.modeshape_animation["mesh"]) is not list:
+                self.modeshape_animation["mesh"] = [self.modeshape_animation["mesh"]]
+
+            for mesh in self.modeshape_animation["mesh"]:
+
+                self.plot.update_coordinates(self.modeshape_animation["or_pts"] + add_val, mesh=mesh,render = False)
+                if self.modeshape_animation["scalars"]:
+                    if self.modeshape_animation["animate_secondary_mode_shape"]==True:
+
+                        self.plot.update_scalars(add_val_secondary, mesh=mesh ,render = False) # for color changing
+                    else: 
+                        self.plot.update_scalars(np.sqrt(np.mean(add_val ** 2, axis=1)).reshape(self.modeshape_animation["or_pts"].shape[0]), mesh=mesh ,render = False) # for color changing
 
             self.plot.render()
             if self.take_gif:
@@ -236,8 +242,9 @@ class view3D():
         Clear mode shape from the 3D display.
         """
 
-        self.plot.update_coordinates(self.modeshape_animation["or_pts"], mesh=self.modeshape_animation["mesh"], render=True)
-        self.plot.update_scalars(np.zeros(self.modeshape_animation["or_pts"].shape[0]), mesh=self.modeshape_animation["mesh"], render=False)
+        for mesh in self.modeshape_animation["mesh"]:
+            self.plot.update_coordinates(self.modeshape_animation["or_pts"], mesh=mesh, render=True)
+            self.plot.update_scalars(np.zeros(self.modeshape_animation["or_pts"].shape[0]), mesh=mesh, render=False)
         self.plot.update_scalar_bar_range(clim=[-100,100])
 
 
@@ -838,13 +845,18 @@ class view3D():
         else:
             self.add_action(self.show_hide_toolbar, "VPs", self.show_hide_vps)
 
-        x = df["Position_1"].unique()
-        y = df["Position_2"].unique()
-        z = df["Position_3"].unique()
+        ind = np.unique(df["Grouping"], return_index=True)[1]
+
+        x = df.iloc[ind]["Position_1"]
+        y = df.iloc[ind]["Position_2"]
+        z = df.iloc[ind]["Position_3"]
         position = np.asarray([x, y, z]).T
         position *= scale
-        vp_mesh,vp_actor = self.add_vp(position,color = color,size = size,**kwargs)
-        self.global_vps.append([vp_mesh, vp_actor])
+
+        for position_ in position:
+            vp_mesh,vp_actor = self.add_vp(position_,color = color,size = size,**kwargs)
+            self.global_vps.append([vp_mesh, vp_actor])
+
         self.vps_visible = True
 
 
@@ -953,19 +965,20 @@ class view3D():
         if self.global_labels == []:
             self.add_action(self.show_hide_toolbar, "Clear Labels", self.clear_labels)
 
-        x = df["Position_1"].unique()
-        y = df["Position_2"].unique()
-        z = df["Position_3"].unique()
+        ind = np.unique(df["Grouping"], return_index=True)[1]
+
+        x = df.iloc[ind]["Position_1"]
+        y = df.iloc[ind]["Position_2"]
+        z = df.iloc[ind]["Position_3"]
         position = np.asarray([x, y, z]).T
         position *= scale
 
-        L = df["Grouping"].unique()
+        L = df.iloc[ind]["Grouping"]
 
         self.plot.add_point_labels(position, L, font_size=font_size,name = name,font_family = "times",shape_opacity=0.5,shape_color = GREEN,show_points=False,**kwargs)
         self.global_labels.append([[position, L], name])
         self.labels_visible = True
 
-
     def show_hide_accelerometers(self):
         """
         Toggles visibility of the accelerometers in the 3D display.
